# periMICA Debian Bullseye Container
## Overview
The Debian Container allows to develop customized applications for periMICA Basesystem. Install the container by logging to periMICA
Web-UI. Next click on `Install` and select the container - tarfile to be uploaded. After installing Start the container by right-clicking
on the Container Icon and select `Start App`. 

## mDNS access:
All periMICA containers provide mDNS publishing to allow easy access of services via name. The syntax of the provided mDNS name is `<container-name>-<periMICA-name>.local`. So if your
container name and periMICA name were `debian` and `periMICA-xxzyy` the container mDNS - Address would be `debian-periMICA-xxzyy.local`. The mDNS address will be resolved by your OS system name resolution services to the link-local IPv6 of your periMICA container. Unless IPv4 Only mode is active for your periMICA. Then it will be resolved to the IPv4 address of your very periMICA.
## Web UI:
The Debian container provides a default Web UI/API for configuring security features of the Container Web Access and also retrieves
OTP information for accessing the container via SSH. It is recommended to configure the corresponding certificates of your 
container and enable Client Certificate Authentication. Take a look on periMICA PKI2go - Container to create your own certificates. 
## API
The following REST API calls are provided:
- `/security` - GET and PATCH, JSON configuration `{"enable_user_role": <boolean>}` to enable / disable client certificate authentication.
- `/security/host-cert` - GET and PATCH the host certificate of the Container Web Server. The host certificate must be passed with the private key appended or prepended.
- `/security/client-cert` - GET and PATCH the client certificate of the Container used to authenticate on other webservices. The client certificate must be passed with the private key appended or prepended.
- `/security/root-cert` - GET and PATCH the root certificate of the Container Web Server.
- `/security/reset` - PATCH the security parameters to default configuration.
- `/security/ssh/otp` - GET JSON `{"otp_config": <unsigned int64>}` for One Time Password.
- `/config` - GET and PATCH, JSON configuration `{"application_name": <string>}` which sets the application name published by mDNS services.
## SSH Access:
Using One-Time-Password protection allows developers to access the debian container via SSH. Go to the Container Web UI and 
click on Create One Time Password. Now use an SSH Client (eg. putty) and enter `<container-name>-<periMICA-name>.local`. If 
you have IPv4-only access enabled on your periMICA make sure to append the corresponding SSH - port to your container mDNS address.
By initializing your SSH connection put in "root" as username and enter the OTP password you received from the container Web UI.

# Developing Customized Web Applications for periMICA
## Go Implementation
### Overview
All debian based containers use an https server implementation (`rootfs/var/www/containerservice/webserver/main.go`) which is connected to an grpc server running on loopback interface (`rootfs/var/www/containerservice/grpc_server/main.go`). The Https Server is running as `web` user whereas the grpc server has full permission. Basically the https server forwards all clients requests to the grpc backend, which is performing the actual functionality. 
### Setup cross compile environment
First you need to install go:
```
curl -O https://dl.google.com/go/go1.17.1.linux-amd64.tar.gz
tar xvf go1.17.1.linux-amd64.tar.gz
chown -R root:root ./go
sudo mv go /usr/local/
echo "export GOROOT=/usr/local/go" >> ~/.profile
echo "export GOPATH=$HOME/work" >> ~/.profile
echo "export GOBIN=$HOME/work/bin" >> ~/.profile
echo "export PATH=$PATH:/usr/local/go/bin:$HOME/work/bin" >> ~/.profile
source ~/.profile
```
After that install the necessary go packages and dependencies:
```
WORK=$HOME/work # adapt this to your needs
mkdir $WORK
git clone -b v1.35.0 https://github.com/grpc/grpc-go $WORK/grpc-go
export PATH="$PATH:$(go env GOPATH)/bin"
curl -LO https://github.com/protocolbuffers/protobuf/releases/download/v3.15.8/protoc-3.15.8-linux-x86_64.zip
go get -u github.com/golang/protobuf/protoc-gen-go
go get -u google.golang.org/grpc/cmd/protoc-gen-go-grpc
unzip protoc-3.15.8-linux-x86_64.zip -d $HOME/.local
export PATH="$PATH:$HOME/.local/bin"
git clone https://github.com/googleapis/googleapis.git $WORK/googleapis
```
After that validate your installation. Therefore initialize your module first.
```
cd (Debian-repo)/rootfs/var/www/containerservice
go mod init containerservice
go mod tidy
```
Then build your services:
```
protoc --go_out=./proto -I$WORK/googleapis/ -Iproto/ --go_opt=paths=source_relative --go-grpc_out=./proto --go-grpc_opt=paths=source_relative proto/*.proto
GOOS=linux GOARCH=arm go build -o go_webserver webserver/main.go
GOOS=linux GOARCH=arm go build -o go_grpc_server grpc_server/main.go
```

### Extend your Custom Container
If you want to create a new debian based container it is recommended to use the existing http server and grpc backend and just extend it by your new methods. Go ahead and copy the debian containerservice files to your custom container repo:
```
mkdir -p (Custom-Container-repo)/rootfs/var/www/containerservice
cp -r (Debian-repo)/rootfs/var/www/containerservice/* (Custom-Container-repo)/rootfs/var/www/containerservice/
```
Change to `(Custom-Container-repo)/rootfs/var/www/containerservice` folder. Everything is now referred relative to this folder.
Create a new proto file, eg. `custom.proto` under `./proto`:
```
syntax = "proto3";
import "google/protobuf/empty.proto";
import "google/api/annotations.proto";
option go_package = "containerservice/proto";
package perinet.api.periCONTAINER.custom;

service PeriContainerCustom {
    rpc GetCustom (google.protobuf.Empty) returns ( Custom ){
        option (google.api.http) = {
            get: "/custom"
        };
    };
}

message Custom {
    string custom_var = 1;
}
```
At next extend the `./grpc_server/main.go`:
```
func (s *customServer) GetCustom(ctx context.Context, in *pb.Empty) (*pb.Custom, error) {
    return &pb.Custom{CustomVar:"my custom string"}, nil
}
```
Also add a typedef after the header in this file and also add `context` to the imports:
```
type customServer struct {
    pb.UnimplementedPeriContainerCustomServer
}
```
And finally register your service in the main:
```
pb.RegisterPeriContainerCustomServer(s, &customServer{})
```
Try to build your code by calling the last operations given under chapter `Setup Environment`. At last the new call needs to be registered by creating a new file `./helper/rest_custom.go`. Add a new rest endpoint by putting:
```
package containerservice

import (
	"net/http"
	"containerservice/helper/api"

	pb "containerservice/proto"
)

func AddCustomEndpoints(em *api.EndpointManager) {
	e := api.New("/custom")
	e.Get = api.RestMethod { api.READER, func(env api.ReqEnv) {
		w := env.Resp
		c:= pb.NewPeriContainerCustomClient(env.GrpcConn)
		m, _:= c.GetCustom(env.GrpcContext, &pb.Empty{})
		w.Header().Set("Content-type", "application/json; charset=utf-8;")
		w.WriteHeader(http.StatusOK)
		w.Write([]byte("{\"custom_var\":\"" + m.CustomVar + "\"}"))
	}}
	*em = em.AddEndpoints([]api.Endpoint{e})
}
```
And call this function within  `./webserver/main.go` by adding
```
containerservice.AddCustomEndpoints(&endpointManager)
```
to the main function. This registers a Http Get Method, that requires Reader rights if client authentication is available. All other Http Methods can be implemented as well by the same signature. The request payload can be extracted by the `env.Payload` variable. See `helper/api/endpoint.go` for more information.

Go ahead and rebuild your applications, after that copy `go_webserver` and `go_gprc_server` to `/usr/bin` on your container rootfs. Make sure to stop your services in your container before by calling:
```
systemctl stop webservice
systemctl stop containerservice
```
respectively.
